<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kwalletmanager" version="3">
  <MenuBar>
    <Menu name="file"><text>&amp;File</text>
      <Action name="wallet_create"/>
      <Separator/>
      <Action name="wallet_merge"/>
      <Action name="wallet_import"/>
      <Action name="wallet_export"/>
      <Action name="wallet_import_encrypted"/>
      <Action name="wallet_export_encrypted"/>
      <Separator/>
      <Action name="close_all_wallets"/>
      <Separator/>
      <Action name="wallet_delete"/>
    </Menu>
    <Menu name="settings"><text>&amp;Settings</text>
      <Action name="wallet_settings" append="configure_merge"/>
    </Menu>
    <Menu name="help"><text>&amp;Help</text>
    </Menu>
  </MenuBar>
</gui>
